#ifndef FILE_RABBITS_SEEN
#define FILE_RABBITS_SEEN

#define MAX_X               100
#define MAX_Y               100
#define MAX_COORD           MAX_X*MAX_Y
#define MAX_HUNGER          10
#define MAX_GRASS           10
#define MIN_BREED_HUNGER    5
#define DIE_HUNGER          0
#define GRASS_GROWTH_RATE   1
/* Coordinate for out of bounds */
#define OOB                 -1

/* #define DEBUG               1 */

#define _ISOC99_SOURCE      1
#define _GNU_SOURCE         1
#define _XOPEN_SOURCE       1

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <stdbool.h>

extern char *program_invocation_short_name;

typedef struct user_args user_args;
struct user_args {
    float       breed_ratio;
    unsigned short int time_steps;
};

/* Neighbors for grassy field, will be OOB if they are off the grid */
typedef struct neighbors neighbors;
struct neighbors {
    int up;
    int down;
    int left;
    int right;
};

/* Coordinate type, both 2D and 1D */
typedef struct coord coord;
struct coord {
    short unsigned int x;
    short unsigned int y;
    unsigned int i;
};

/* Field of grass and rabbits */
typedef struct grassy_field grassy_field;
struct grassy_field {
    /* rabbit will vary from 0 (no rabbit) to 10 (fat rabbit) */
    unsigned short int rabbit;
    /* grass will vary from 0 to 10 */
    unsigned short int grass;
    struct neighbors neighbors;
};

void usage();

struct user_args get_user_args(const int,char **);

struct grassy_field *init_field(
        const struct user_args,
        struct drand48_data *
        );

unsigned int calc_2d21d(
        const unsigned int, // dims
        const unsigned int, // x
        const unsigned int // y
);

int breed_rabbits (
        const struct user_args,
        struct grassy_field *,
        struct drand48_data *
        );

bool try_move(
        struct grassy_field *,
        const unsigned int, // x
        const unsigned int, // y
        const unsigned int, // 1d
        struct drand48_data *
        );

unsigned short int adj_empty_cells(
        const struct grassy_field *,
        const unsigned int,  // 1d
        unsigned int *
        );

void print_grid(
        const struct grassy_field *,
        const unsigned int // timestep
        );

void move_rabbit(
        struct grassy_field *,
        const unsigned int, // old cell
        const unsigned int // new cell
        );

int rand_adj_empty_cell(
        const struct grassy_field *,
        const unsigned int, // 1d
        struct drand48_data *
        );

bool breed_rabbit(
        const struct user_args,
        struct grassy_field *,
        const unsigned int, // x
        const unsigned int, // y
        const unsigned int, // 1d coord
        struct drand48_data *
        );

#endif /* !FILE_RABBITS_SEEN */
